//*****************************************************************************
// Description     : Main sample program of list mode
// Target Device   : APV8216
// Target Platform : Linux, C
// Date            : Jul 2023
// Version         : 2.2
//*****************************************************************************
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <errno.h>

#include "sub_sitcp.h"

//
// Modify the following defines according to your environment
//
#define DEVICE_IPADDR     "192.168.10.128"  // IP address of target device
#define LIST_FILENAME    "./list_data.bin"  // name of the file where list data saved
#define MEASUREMENT_TIME_MSEC       (5000)  // measurement time (msec)

#define IPADDR_OWNPC             "0.0.0.0"  // IP address of own PC; Normally leave 0.0.0.0
#define DEVICE_PORT_CTRL_OWNPC     (50000)  // source port number for command
#define DEVICE_SOCKBUF_CMD     (16 * 1024)  // for command communication(16kB)
#define DEVICE_SOCKBUF_DATA  (1024 * 1024)  // for receiving data(1MB)
#define LIST_TRANSFER_LEN          (16000)  // length of list data per transfer

//
// Specific to device
//
#define DEVICE_PORT_DATARECV    (24)  // port number for data reception
#define DEVICE_CNT_PER_NSEC     (10)  // 1count = 10nsec
#define DEVICE_OFFSETADDR_CH (0x100)  // command address offset per CH
#define NUM_OF_CH               (16)  // number of CH

//
// Static global variables
//
static int cmd_socket;	// socket for command communication
static int data_socket;	// socket for data transfer

//#########################################################################
//  Name      : main
//  Function  : entry point
//  Arguments :
//      in argc : number of parameters
//      in argv : parameters
//  Return :
//      0 : success
//      non-0 : failure
//  Note : 
//#########################################################################
int main(int argc, char **argv)
{
	int err_cnt;
	int ret;
	unsigned long long mtm_count;
	struct sockaddr_in saddr;
	int optval;
	unsigned long long now_count;
	unsigned int now_msec;
	unsigned int output_cps[NUM_OF_CH];
	FILE *fp;
	char list_data[LIST_TRANSFER_LEN];
	int ch;
	int recv_cnt, total_list_bytes;

	// convert msec to count
	mtm_count = (((unsigned long long)(MEASUREMENT_TIME_MSEC) * 1000000UL)
				/ (unsigned long long)(DEVICE_CNT_PER_NSEC));

	// ===========================================================================
	// prepare UDP socket for command communication
	cmd_socket = socket(AF_INET, SOCK_DGRAM, 0);
	if (cmd_socket > 0) { // socket created
		// change maximum socket receive buffer size
		optval = DEVICE_SOCKBUF_CMD;
		setsockopt(cmd_socket, SOL_SOCKET, SO_RCVBUF, &optval, sizeof(optval));

		// assign own address to the socket
		memset(&saddr, 0, sizeof(saddr));
		saddr.sin_family = AF_INET;
		saddr.sin_port = htons(DEVICE_PORT_CTRL_OWNPC);  // port number for receiving cmd reply
		saddr.sin_addr.s_addr = inet_addr(IPADDR_OWNPC); // IP address of own PC
		ret = bind(cmd_socket, (struct sockaddr*)(&saddr), sizeof(saddr));
		if (ret != 0) {
			printf("bind error(%d)\n", errno);
			close(cmd_socket);
		}
	}
	else {
		printf("create UDP socket error(%d)\n", errno);
		return(1);
	}

	// ===========================================================================
	// prepare TCP socket for data transfer
	data_socket = socket(AF_INET, SOCK_STREAM, 0);
	if (data_socket > 0) { // socket created
		// change maximum socket receive buffer size
		optval = DEVICE_SOCKBUF_DATA;
		setsockopt(data_socket, SOL_SOCKET, SO_RCVBUF, &optval, sizeof(optval));

		// connect to target device
		memset(&saddr, 0, sizeof(saddr));
		saddr.sin_family = AF_INET;
		saddr.sin_port = htons(DEVICE_PORT_DATARECV);     // port number for receiving data
		saddr.sin_addr.s_addr = inet_addr(DEVICE_IPADDR); // IP address of target device
		ret = connect(data_socket, (struct sockaddr*)(&saddr), sizeof(saddr));
		if (ret != 0) {
			printf("connect error(%d)\n", errno);
			close(data_socket);

			close(cmd_socket);
		}
	}
	else {
		printf("create TCP socket error(%d)\n", errno);
		close(cmd_socket);
		return(2);
	}

	// ===========================================================================
	// set parameters to the device

	err_cnt = 0;

	// ADC gain(0:16384, 1:8192, 2:4096, 3:2048, 4:1024, 5:512, 6:256)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000114, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000214, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000314, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000414, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000514, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000614, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000714, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000814, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000914, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A14, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B14, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C14, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D14, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E14, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F14, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4001014, 0);
	if (ret < 0) err_cnt++;

	// slow threshold(0 to 16383)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000112, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000212, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000312, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000412, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000512, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000612, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000712, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000812, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000912, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A12, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B12, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C12, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D12, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E12, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F12, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4001012, 100);
	if (ret < 0) err_cnt++;

	// LLD(0 to 16383)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400011C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400021C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400031C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400041C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400051C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400061C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400071C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400081C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400091C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A1C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B1C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C1C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D1C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E1C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F1C, 100);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400101C, 100);
	if (ret < 0) err_cnt++;

	// ULD(0 to 16383)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400011E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400021E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400031E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400041E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400051E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400061E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400071E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400081E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400091E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A1E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B1E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C1E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D1E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E1E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F1E, 16380);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400101E, 16380);
	if (ret < 0) err_cnt++;

	// peak detect mode(0:abs, 1:fast)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400013E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400023E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400033E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400043E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400053E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400063E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400073E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400083E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400093E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A3E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B3E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C3E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D3E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E3E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F3E, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB400103E, 0);
	if (ret < 0) err_cnt++;

	// initial offset(adjusted value board by board)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000140, 96);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000240, 107);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000340, 121);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000440, 112);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000540, 212);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000640, 135);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000740, 126);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000840, 146);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000940, 144);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A40, 149);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B40, 145);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C40, 181);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D40, 108);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E40, 98);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F40, 142);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4001040, 163);
	if (ret < 0) err_cnt++;

	// user offset(-32767 to 32767)
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000142, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000242, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000342, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000442, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000542, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000642, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000742, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000842, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000942, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000A42, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000B42, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000C42, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000D42, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000E42, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000F42, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4001042, 0);
	if (ret < 0) err_cnt++;

	// clear data
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000040, 0);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000040, 1);
	if (ret < 0) err_cnt++;
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000040, 0);
	if (ret < 0) err_cnt++;

	// measurement mode: list
	ret = sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000010, 1);
	if (ret < 0) err_cnt++;

	// measurement time
	ret = sub_sitcp_write_6byte(DEVICE_IPADDR, cmd_socket, 0xB4000016, mtm_count);
	if (ret < 0) err_cnt++;

	if (err_cnt > 0) {
		printf("config error(%d)...\n", err_cnt);
		close(cmd_socket);
		close(data_socket);
		return(3);
	}

	// ===========================================================================
	// open file in advance
	fp = fopen(LIST_FILENAME, "wb");
	if (fp == NULL) {
		printf("Failed to create file(%s)\n", LIST_FILENAME);
		close(cmd_socket);
		close(data_socket);
		return(4);
	}

	// ===========================================================================
	// start measurement

	// clear data once
	sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000040, 0);
	sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000040, 1);
	sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000040, 0);

	// wait for completion of clear processing
	usleep(100*1000);

	// stop once, then start
	sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000014, 0);
	sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000014, 1);

	printf("start measurement!!!\n");
    printf("<time(msec)>   |<output rate(cps)>\n");
    printf(" real/total    |  CH1     CH2     CH3     CH4     CH5     CH6     CH7     CH8     CH9     CH10    CH11    CH12    CH13    CH14    CH15   CH16\n");
    printf("---------------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------+-------\n");

	// ===========================================================================
	// main measuremet loop
	recv_cnt = 0;
	total_list_bytes = 0;
	while(1) {
		if (recv_cnt % 500 == 0) { // read status every 500 receiving list

			// read throughput count rate(cps) of each CH: 4byte value
			for (ch = 0; ch < NUM_OF_CH; ch++) {
				output_cps[ch] = sub_sitcp_read_4byte(DEVICE_IPADDR, cmd_socket,
													0xB400012C + DEVICE_OFFSETADDR_CH * ch,
													&ret); 
				if (ret != 0) break; // abort for-loop
			}
			if (ret != 0) {
				printf("<WARNING!> Failed to read throughput count rate(cps) of CH%d (errno=%d)\n", ch+1, ret);
			}

			// read elapsed time: 6byte value
			now_count = sub_sitcp_read_6byte(DEVICE_IPADDR, cmd_socket, 0xB400001C, &ret);
			if (ret != 0) {
				printf("<WARNING!> Failed to read elapesd time (errno=%d)\n", ret);
			} else {
				// convert count to msec
				now_msec = (unsigned int)((now_count * (unsigned long long)(DEVICE_CNT_PER_NSEC)) / 1000000UL);

				// display status
				printf("%5d/%5d    |", now_msec, (unsigned int)(MEASUREMENT_TIME_MSEC));
				printf("%6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d\n",
						output_cps[0], output_cps[1], output_cps[2], output_cps[3],
						output_cps[4], output_cps[5], output_cps[6], output_cps[7],
						output_cps[8], output_cps[9], output_cps[10], output_cps[11],
						output_cps[12], output_cps[13], output_cps[14], output_cps[15]);

				// break a loop when reached to measurement time
				if (now_count >= mtm_count) {
					break;
				}
			}
		}

		// receive and write list data to file
		ret = recv(data_socket, list_data, LIST_TRANSFER_LEN, MSG_DONTWAIT);
		if (ret > 0) {
			fwrite(list_data, (size_t)1, (size_t)ret, fp);
			total_list_bytes += ret;
		}
		recv_cnt++;

		// wait a while
		usleep(1000);
	} // end of measurement loop
	fclose(fp);

	if (ret >= 0) {
		printf("measurement completed!!!\n");
	}
	else {
		printf("measurement error(%d)...\n", ret);
	}

	if (total_list_bytes > 0) printf("total list size = %d bytes\n", total_list_bytes);

	// stop measurement
	sub_sitcp_write_2byte(DEVICE_IPADDR, cmd_socket, 0xB4000014, 0);

	// close sockets
	close(cmd_socket);
	close(data_socket);

	return(0);
}
